@@ -1,9 +1,11 @@
 from __future__ import print_function
 import os
+import sys
 from optparse import OptionParser
 
 from unilint.__version__ import VERSION
 from unilint.common import UnilintException
+from unilint.common_source_plugin import COMMON_SOURCE
 from unilint.unilint_plugin import UnilintPluginInitException
 from unilint.custom_format import FLAGS, print_formatted
 
@@ -103,37 +105,40 @@ def order_issues(issues):
     return sorted(issues, key=keyfun)
 
 
-def check_path(path, basepath, options, subdirs=None, files=None, cleared_paths=None, is_folder=True):
+def check_path(abspath, basepath, options, subdirs=None, files=None, is_folder=True):
     issues = []
     categorized_resources = {}
-    if cleared_paths is None:
-        cleared_paths = {}
 
     for plugin_id in options.selected_plugins:
-        new_categories = options.selected_plugins[plugin_id].categorize_type(options, path, subdirs, files)
+        new_categories = options.selected_plugins[plugin_id].categorize_type(options, abspath, subdirs, files)
         extend_maybe(categorized_resources, new_categories)
 
     for resource, categories in categorized_resources.items():
-        if 'generated' in categories:
-            if not options.check_generated:
-                continue
-        if 'test' in categories:
-            if not options.check_tests:
-                continue
+        if abspath == resource:
+            if 'generated' in categories:
+                if not options.check_generated:
+                    return None
+            if 'test' in categories:
+                if not options.check_tests:
+                    return None
+            if 'doc' in categories:
+                if not options.check_docs:
+                    return None
+
+    for resource, categories in categorized_resources.items():
+        if 'hidden' in categories:
+            continue
+        if 'backup' in categories:
+            continue
         if options.debug:
             print("processing: %s as %s"%(resource, categories))
         for plugin_id in options.selected_plugins:
             result = options.selected_plugins[plugin_id].check_resource(options, resource, categories)
             extend_maybe(issues, result)
-            if result is not None and is_folder:
-                if not plugin_id in cleared_paths:
-                    cleared_paths[plugin_id] = [path]
-                else:
-                    cleared_paths[plugin_id].append(path)
             if result is not None and not options.ordered and not options.raw:
                 # get the output to the user as quickly as possible
                 print_issues(options, result, basepath)
-    return issues, cleared_paths
+    return issues
 
 
 def run_cmd(path, options, args):
@@ -141,6 +146,11 @@ def run_cmd(path, options, args):
         print('\n'.join(PLUGINS.keys()))
         return 0
 
+    # if common is not selected, select it. can still be deselected.
+    if options.selected_plugins is not None:
+        if not COMMON_SOURCE in options.selected_plugins:
+            options.selected_plugins += ',%s'%COMMON_SOURCE
+
     if options.version:
         options.selected_plugins = resolve_plugins(options.selected_plugins,
                                                    options.deselected_plugins,
@@ -167,45 +177,37 @@ def run_cmd(path, options, args):
         if len(args) == 0:
             raise UnilintException('Unable to use cwd as path')
         else:
-            raise UnilintException('Unable to run on %s'%args[0])
+            raise UnilintException('Unable to run on path %s'%args[0])
 
+    path = os.path.abspath(path)
     if os.path.isdir(path):
         issues = []
-        ignoreddirs = []
-        cleared_paths = {} # stores for each plugin the parent folders this plugin is done with
 
         for (parentdir, subdirs, files) in os.walk(path):
+            ignoreddirs = []
+            for dirname in subdirs:
+                if (dirname in ['build', '.svn', 'CVS', '.hg', '.git']
+                    or 'egg-info' in dirname):
 
-            if os.path.basename(parentdir) in ['build', '.svn', 'CVS', '.hg', '.git']:
-                # inserting, to make string search faster
-                ignoreddirs.insert(0, parentdir)
+                    ignoreddirs.append(dirname)
+            for dirname in ignoreddirs:
+                subdirs.remove(dirname)
+            if len(files) == 0 and len(subdirs) == 0:
                 continue
-            if 'egg-info' in parentdir:
-                # inserting, to make string search faster
-                ignoreddirs.insert(0, parentdir)
-                continue
-
-            ignore = False
-            for ignored in ignoreddirs:
-                if parentdir.startswith(ignored):
-                    ignore = True
-                    break
-            if ignore is True:
-                continue
-            if len(files) == 0 and len(subdirs)==0:
-                continue
-
-            new_issues, cleared_paths = check_path(parentdir, path, options, subdirs, files, cleared_paths=cleared_paths)
-            if not options.raw:
-                extend_maybe(issues, new_issues)
-                if options.ordered:
-                    dir_issues = order_issues(new_issues)
-                    print_issues(options, dir_issues, path)
+            new_issues = check_path(parentdir, path, options, subdirs, files)
+            extend_maybe(issues, new_issues)
+            if not options.raw and options.ordered:
+                # print here unless we already printed in check_path
+                dir_issues = order_issues(new_issues)
+                print_issues(options, dir_issues, path)
     else:
         # is file
-        issues, _ = check_path(path, os.path.dirname(path), options=options, is_folder=False)
-        print_issues(options, issues, path)
+        issues = check_path(path, os.path.dirname(path), options=options, is_folder=False)
+        if not options.raw and options.ordered:
+            print_issues(options, issues, path)
 
+    # write to std_err so that report on std_out remains unchanged
+    sys.stderr.write("%s issues found\n"%len(issues or []))
     return 0
 
 
@@ -235,6 +237,9 @@ def get_unilint_parser(usage):
     parser.add_option("-t", "--tests", dest="check_tests", default=False,
                       help="Also check unit tests",
                       action="store_true")
+    parser.add_option("--docs", dest="check_docs", default=False,
+                      help="Also check doc files",
+                      action="store_true")
     parser.add_option("-g", "--generated", dest="check_generated", default=False,
                       help="Also check generated Files",
                       action="store_true")
@@ -284,7 +289,7 @@ def unilint_main(argv):
     prog = argv[0]
     args = argv[1:]
 
-    parser = get_unilint_parser("usage: %s [OPTIONS] [PATH]"%prog)
+    parser = get_unilint_parser("usage: %s [OPTIONS] [PATH]"%os.path.basename(prog))
     (options, args) = parser.parse_args(args)
 
     options, args, path = evaluate_options(options, args)
